@@ -46,7 +46,9 @@ NULL
 #'   counts (column named 'TOTAL') will be amended to the \code{data.table} that
 #'   is returned.
 #' @param progress Logical, whether to show progress bar.
-#' @param ... Further arguments.
+#' @param ... Further arguments. If \code{.Object} is a \code{remote_corpus}
+#'   object, the three dots (\code{...}) are used to pass arguments. Hence, it is
+#'   necessary to state the names of all arguments to be passed explicity.
 #' @return A \code{data.table} if argument query is used, a \code{count}-object,
 #'   if query is \code{NULL} and \code{.Object} is a character vector (referring 
 #'   to a corpus) or a \code{partition}, a \code{count_bundle}-object, if \code{.Object}
@@ -86,6 +88,29 @@ setMethod("count", "partition", function(
   decode = TRUE, p_attribute = getOption("polmineR.p_attribute"),
   mc = getOption("polmineR.cores"), verbose = TRUE, progress = FALSE,
   ...
+  )
+  callNextMethod()
+)
+
+
+#' @rdname count-method
+setMethod("count", "subcorpus", function(
+  .Object, query = NULL, cqp = is.cqp, check = TRUE, breakdown = FALSE,
+  decode = TRUE, p_attribute = getOption("polmineR.p_attribute"),
+  mc = getOption("polmineR.cores"), verbose = TRUE, progress = FALSE,
+  ...
+)
+  callNextMethod()
+)
+
+
+
+#' @rdname count-method
+setMethod("count", "slice", function(
+  .Object, query = NULL, cqp = is.cqp, check = TRUE, breakdown = FALSE,
+  decode = TRUE, p_attribute = getOption("polmineR.p_attribute"),
+  mc = getOption("polmineR.cores"), verbose = TRUE, progress = FALSE,
+  ...
 ){
   
   if ("pAttribute" %in% names(list(...))) p_attribute <- list(...)[["pAttribute"]]
@@ -101,7 +126,11 @@ setMethod("count", "partition", function(
           if (is.null(cposHits)) return( NULL )
           hitsString <- apply(
             cposHits, 1,
-            function(x) paste(CQI$cpos2str(.Object@corpus, p_attribute, x[1]:x[2]), collapse = ' ')
+            function(x)
+              paste(
+                cl_cpos2str(corpus = .Object@corpus, p_attribute = p_attribute, cpos = x[1]:x[2], registry = registry()),
+                collapse = ' '
+              )
           )
           result <- table(hitsString)
           dt <- data.table(query = x, match = names(result), count = as.vector(unname(result)))
@@ -118,22 +147,22 @@ setMethod("count", "partition", function(
       )
       return( rbindlist(dts) )
     } else if (breakdown == FALSE){
-      .getNumberOfHits <- function(query, partition, cqp, p_attribute, ...) {
+      .fn <- function(query, obj, cqp, p_attribute, ...) {
         .message("processing query", query, verbose = verbose)
-        cposResult <- cpos(.Object = .Object, query = query, cqp = cqp, check = check, p_attribute = p_attribute, verbose = FALSE)
+        cposResult <- cpos(.Object = obj, query = query, cqp = cqp, check = check, p_attribute = p_attribute, verbose = FALSE)
         if (is.null(cposResult)) return( 0 ) else return( nrow(cposResult) )
       }
-      no <- as.integer(blapply(
+      no <- as.integer(unlist(blapply(
         as.list(query),
-        f = .getNumberOfHits,
-        partition = .Object, cqp = cqp, p_attribute = p_attribute,
+        f = .fn,
+        obj = .Object, cqp = cqp, p_attribute = p_attribute,
         mc = mc, verbose = verbose, progress = progress
-      ))
+      )))
       return( data.table(query = query, count = no, freq = no/.Object@size) )
     }
   } else {
     pAttr_id <- paste(p_attribute, "id", sep = "_")
-    if (length(p_attribute) == 1){
+    if (length(p_attribute) == 1L){
       countMatrix <- RcppCWB::region_matrix_to_count_matrix(
         corpus = .Object@corpus, p_attribute = p_attribute,
         matrix = .Object@cpos
@@ -142,7 +171,7 @@ setMethod("count", "partition", function(
       setnames(TF, old = c("V1", "V2"), new = c(pAttr_id, "count"))
     } else {
       cpos <- unlist(apply(.Object@cpos, 1, function(x) x[1]:x[2]))
-      idList <- lapply(p_attribute, function(p) CQI$cpos2id(.Object@corpus, p, cpos))
+      idList <- lapply(p_attribute, function(p) cl_cpos2id(corpus = .Object@corpus, p_attribute = p, cpos = cpos, registry = registry()))
       names(idList) <- paste(p_attribute, "id", sep = "_")
       ID <- data.table::as.data.table(idList)
       setkeyv(ID, cols = names(idList))
@@ -153,8 +182,8 @@ setMethod("count", "partition", function(
       dummy <- lapply(
         1L:length(p_attribute),
         function(i){
-          str <- as.nativeEnc(CQI$id2str(.Object@corpus, p_attribute[i], TF[[pAttr_id[i]]]), from = .Object@encoding)
-          TF[, eval(p_attribute[i]) := str , with = TRUE] 
+          str <- cl_id2str(corpus = .Object@corpus, p_attribute = p_attribute[i], id = TF[[pAttr_id[i]]], registry = registry())
+          TF[, eval(p_attribute[i]) := as.nativeEnc(str, from = .Object@encoding) , with = TRUE] 
         })
       setcolorder(TF, neworder = c(p_attribute, pAttr_id, "count"))
     } else {
@@ -184,7 +213,7 @@ setMethod("count", "partition_bundle", function(.Object, query = NULL, cqp = FAL
     .message("getting hits for query/queries", verbose = verbose)
     DT <- hits(.Object, query = query, cqp = cqp, p_attribute = p_attribute, mc = mc, progress = progress, verbose = verbose)@stat
     .message("rearranging table", verbose = verbose)
-    DT_cast <- dcast.data.table(DT, partition~query, value.var = "count", fill = 0)
+    DT_cast <- dcast.data.table(DT, partition ~ query, value.var = "count", fill = 0)
     
     # remove counts that are not in one of the partitions
     noPartition <- which(is.na(DT_cast[["partition"]]) == TRUE)
@@ -210,87 +239,101 @@ setMethod("count", "partition_bundle", function(.Object, query = NULL, cqp = FAL
       }
     }
     
-    if (total){
-      .message("adding total number of hits (col 'TOTAL', verbose = verbose)")
-      DT_cast[, "TOTAL" := rowSums(DT_cast[, 2L:ncol(DT_cast), with = FALSE]), with = TRUE]
-    }
+    if (total) DT_cast[, "TOTAL" := rowSums(DT_cast[, 2L:ncol(DT_cast), with = FALSE]), with = TRUE]
     return(DT_cast)
   } else {
-    corpus <- corpus(.Object)
-    if (length(corpus) > 1) stop("partitions in partition_bundle must be derived from the same corpus")
-    if (verbose) message("... unfolding corpus positions")
-    cpos_list <- lapply(
-      .Object@objects,
-      function(x) data.table(name = x@name, cpos = cpos(x@cpos))
-    )
-    DT <- rbindlist(cpos_list)
-    rm(cpos_list)
+    
+    corpus <- get_corpus(.Object)
+    if (length(corpus) > 1L) stop("partitions in partition_bundle must be derived from the same corpus")
+    
+    if (verbose) message("... creating data.table with corpus positions")
+    cpos_dt <- data.table(do.call(rbind, lapply(.Object@objects, slot, name = "cpos")))
+    cpos_dt[, "name" := do.call(
+      c,
+      lapply(seq_along(.Object@objects), function(i) rep(x = names(.Object@objects)[[i]], times = nrow(.Object@objects[[i]]@cpos)))
+      )]
+    DT <- cpos_dt[, {do.call(c, lapply(1L:nrow(.SD), function(i) .SD[[1]][i]:.SD[[2]][i]))}, by = "name"]
+    setnames(DT, old = "V1", new = "cpos")
+    rm(cpos_dt)
+    
     if (verbose) message(sprintf("... adding ids for p-attribute '%s'", p_attribute))
-    DT[["id"]] <- CQI$cpos2id(corpus = corpus, p_attribute = p_attribute, cpos = DT[["cpos"]])
+    DT[, "id" :=  cl_cpos2id(corpus = corpus, p_attribute = p_attribute, cpos = DT[["cpos"]], registry = registry())]
     if (verbose) message("... performing count")
     CNT <- DT[,.N, by = c("name", "id")]
     rm(DT)
     setnames(CNT, old = "N", new = "count")
+    
     if (verbose) message("... adding decoded p-attribute")
-    CNT[[p_attribute]] <- CQI$id2str(corpus = corpus, p_attribute = p_attribute, id = CNT[["id"]])
+    str_raw <- cl_id2str(corpus = corpus, p_attribute = p_attribute, id = CNT[["id"]], registry = registry())
+    enc <- .Object@objects[[1]]@encoding
+    CNT[, eval(p_attribute) := if (localeToCharset()[1] == enc) str_raw else as.nativeEnc(str_raw, from = enc)]
+    rm(str_raw)
+    
     if (verbose) message("... creating bundle of count objects")
     CNT_list <- split(CNT, by = "name")
     rm(CNT)
-    y <- lapply(
-      1L:length(CNT_list),
-      function(i){
-        new(
-          "count",
-          corpus = corpus,
-          encoding = .Object@objects[[i]]@encoding,
-          p_attribute = p_attribute,
-          stat = CNT_list[[i]],
-          name = names(CNT_list)[[i]],
-          size = size(.Object@objects[[i]])
-        )
-      }
-    )
-    names(y) <- names(.Object)
-    y2 <- new(
-      Class = "count_bundle",
-      objects = y,
-      p_attribute = p_attribute,
-      corpus = corpus,
-      encoding = unique(unlist(lapply(as.list(.Object), function(x) encoding(x))))
-    )
-    return( y2 )
+    y <- new(Class = "count_bundle", p_attribute = p_attribute, corpus = corpus, encoding = enc)
+    .fn <- function(i){
+      new(
+        "count",
+        corpus = corpus,
+        encoding = .Object@objects[[i]]@encoding,
+        p_attribute = p_attribute,
+        stat = CNT_list[[i]],
+        name = .Object@objects[[ CNT_list[[i]][["name"]][[1]] ]]@name,
+        size = size(.Object@objects[[i]])
+      )
+    }
+    y@objects <- if (progress) pblapply(seq_along(CNT_list), .fn) else lapply(seq_along(CNT_list), .fn)
+    names(y@objects) <- names(.Object)
+    return( y )
   }
 })
 
 #' @rdname count-method
-setMethod("count", "character", function(.Object, query = NULL, cqp = is.cqp, check = TRUE, p_attribute = getOption("polmineR.p_attribute"), breakdown = FALSE, sort = FALSE, decode = TRUE, verbose = TRUE, ...){
+setMethod("count", "corpus", function(.Object, query = NULL, cqp = is.cqp, check = TRUE, p_attribute = getOption("polmineR.p_attribute"), breakdown = FALSE, sort = FALSE, decode = TRUE, verbose = TRUE, ...){
 
   if ("pAttribute" %in% names(list(...))) p_attribute <- list(...)[["pAttribute"]]
   
-  stopifnot(.Object %in% CQI$list_corpora())
   if (is.null(query)){
     if (length(p_attribute) == 1L){
-      cnt_file <- file.path(
-        registry_get_home(.Object, registry = Sys.getenv("CORPUS_REGISTRY")),
-        sprintf("%s.corpus.cnt", p_attribute)
+      cnt_file <- file.path(.Object@data_dir, sprintf("%s.corpus.cnt", p_attribute)
       )
       if (file.exists(cnt_file)){
         cnt <- readBin(con = cnt_file, what = integer(), size = 4L, n = file.info(cnt_file)$size, endian = "big")
         TF <- data.table(count = cnt)
       } else {
-        TF <- data.table(count = RcppCWB::get_count_vector(corpus = .Object, p_attribute = p_attribute))
+        TF <- data.table(
+          count = RcppCWB::get_count_vector(corpus = .Object@corpus, p_attribute = p_attribute)
+          )
       }
-      TF[, "id" := 0L:(nrow(TF) - 1L), with = TRUE]
-      setnames(TF, old = "id", new = paste(p_attribute, "id", sep = "_"))
-      if (decode == FALSE){
-        setkeyv(TF, paste(p_attribute, "id", sep = "_"))
-        setcolorder(TF, c(paste(p_attribute, "id", sep = "_"), "count"))
+      p_attr_col_id <- paste(p_attribute, "id", sep = "_")
+      TF[, (p_attr_col_id) := 0L:(nrow(TF) - 1L)]
+
+      if (!decode){
+        setkeyv(TF, p_attr_col_id)
+        setcolorder(TF, c(p_attr_col_id, "count"))
       } else {
-        TF[, "token" := as.nativeEnc(CQI$id2str(.Object, p_attribute, 0L:(nrow(TF) - 1L)), from = registry_get_encoding(.Object)), with = TRUE]
+        # somewhat surprisingly, cl_id2str a good deal faster than reading the
+        # lexicon file directly using readBin a follows:
+        # 
+        # lexicon_file <- file.path(.Object@data_dir, paste(p_attribute, "lexicon", sep = "."))
+        # lexicon_file_size <- file.info(lexicon_file)[["size"]]
+        # tokens <- readBin(con = lexicon_file, what = character(), n = lexicon_file_size)
+        # 
+        # Using a Rcpp implementation for reading in the lexion file might be a
+        # fast solution, see https://gist.github.com/hadley/6353939
+        tokens <- cl_id2str(
+          corpus = .Object@corpus,
+          p_attribute = p_attribute,
+          id = TF[[p_attr_col_id]],
+          registry = registry()
+        )
+        TF[, "token" := as.nativeEnc(tokens, from = encoding(.Object)), with = TRUE] # recode is slow
         Encoding(TF[["token"]]) <- "unknown"
         setnames(TF, old = "token", new = p_attribute)
-        setkeyv(TF, p_attribute)
-        setcolorder(TF, c(p_attribute, paste(p_attribute, "id", sep = "_"), "count"))
+        setkeyv(TF, p_attribute) # slow
+        setcolorder(TF, c(p_attribute, p_attr_col_id, "count"))
         if (sort) setorderv(TF, cols = p_attribute)
       }
     } else {
@@ -298,9 +341,9 @@ setMethod("count", "character", function(.Object, query = NULL, cqp = is.cqp, ch
       tokenStreamDT <- data.table::as.data.table(
         li <- lapply(
           setNames(p_attribute, paste(p_attribute, "id", sep = "_")),
-          function(pAttr){
-            .message("getting token stream for p-attribute: ", pAttr, verbose = verbose)
-            CQI$cpos2id(.Object, pAttr, 0L:(size(.Object) - 1L))
+          function(p_attr){
+            .message("getting token stream for p-attribute: ", p_attr, verbose = verbose)
+            cl_cpos2id(corpus = .Object@corpus, p_attribute = p_attr, 0L:(size(.Object) - 1L), registry = registry())
           }
         )
       )
@@ -310,43 +353,42 @@ setMethod("count", "character", function(.Object, query = NULL, cqp = is.cqp, ch
       if (decode){
         for (pAttr in p_attribute){
           .message("decode p-attribute: ", pAttr, verbose = verbose)
-          TF[, eval(pAttr) := as.nativeEnc(CQI$id2str(.Object, pAttr, TF[[paste(pAttr, "id", sep = "_")]]), from = registry_get_encoding(.Object)), with = TRUE]
+          TF[, eval(pAttr) := as.nativeEnc(cl_id2str(corpus = .Object@corpus, p_attribute = pAttr, id = TF[[paste(pAttr, "id", sep = "_")]], registry = registry()), from = encoding(.Object)), with = TRUE]
         }
         setcolorder(TF, c(p_attribute, paste(p_attribute, "id", sep = "_"), "count"))
       }
     }
     y <- new(
       Class = "count",
-      corpus = .Object,
+      corpus = .Object@corpus,
       p_attribute = p_attribute,
-      encoding = registry_get_encoding(.Object),
+      encoding = encoding(.Object),
       stat = TF,
       name = character(),
       size = size(.Object)
     )
     return(y)
   } else {
-    stopifnot(.Object %in% CQI$list_corpora())
-    total <- CQI$attribute_size(.Object, p_attribute, type = "p")
+    total <- cl_attribute_size(corpus = .Object@corpus, attribute = p_attribute, attribute_type = "p", registry = registry())
     if (class(cqp) == "function") cqp <- cqp(query)
     if (length(cqp) > 1) stop("length of cqp is larger than 1, it needs to be 1")
     if (cqp == FALSE){
-      query <- as.corpusEnc(query, corpusEnc = registry_get_encoding(.Object))
+      query <- as.corpusEnc(query, corpusEnc = encoding(.Object))
       count <- sapply(
         query,
         function(query){
-          query_id <- CQI$str2id(.Object, p_attribute, query)
+          query_id <- cl_str2id(corpus = .Object@corpus, p_attribute = p_attribute, str = query, registry = registry())
           # if there is no id for query, query_id will be -5
-          if (query_id >= 0) CQI$id2freq(.Object, p_attribute, query_id) else 0
+          if (query_id >= 0) cl_id2freq(corpus = .Object@corpus, p_attribute = p_attribute, id = query_id, registry = registry()) else 0
         }
       )
       return(data.table(query = query, count = count, freq = count / total))
     } else if (cqp){
-      if (breakdown == FALSE){
+      if (!breakdown){
         count <- sapply(
           query,
           function(query){
-            cpos_matrix <- cpos(.Object, query, cqp = cqp, check = check, p_attribute = p_attribute, encoding = registry_get_encoding(.Object))
+            cpos_matrix <- cpos(.Object@corpus, query, cqp = cqp, check = check, p_attribute = p_attribute)
             if (!is.null(cpos_matrix)){
               return( nrow(cpos_matrix) )
             } else {
@@ -356,7 +398,7 @@ setMethod("count", "character", function(.Object, query = NULL, cqp = is.cqp, ch
           })
         return( data.table(query = query, count = count, freq = count / total) )
       } else {
-        C <- Corpus$new(.Object)
+        C <- Corpus$new(.Object@corpus)
         C$p_attribute <- p_attribute
         retval <- count(.Object = C$as.partition(), query = query, cqp = cqp, p_attribute = p_attribute, breakdown = TRUE)
         return( retval )
@@ -365,6 +407,24 @@ setMethod("count", "character", function(.Object, query = NULL, cqp = is.cqp, ch
   }
 })
 
+
+#' @rdname count-method
+setMethod("count", "character", function(.Object, query = NULL, cqp = is.cqp, check = TRUE, p_attribute = getOption("polmineR.p_attribute"), breakdown = FALSE, sort = FALSE, decode = TRUE, verbose = TRUE, ...){
+  count(
+    .Object = corpus(.Object),
+    query = query,
+    cqp = cqp,
+    check = check,
+    p_attribute = p_attribute,
+    breakdown = breakdown,
+    sort = sort,
+    decode = decode,
+    verbose = verbose,
+    ...
+  )
+})
+
+
 #' @rdname context-class
 setMethod("count", "context", function(.Object) .Object@count )
 
@@ -373,10 +433,10 @@ setMethod("count", "vector", function(.Object, corpus, p_attribute, ...){
   
   if ("pAttribute" %in% names(list(...))) p_attribute <- list(...)[["pAttribute"]]
 
-  ids <- CQI$cpos2id(corpus, p_attribute, .Object)
+  ids <- cl_cpos2id(corpus = corpus, p_attribute = p_attribute, cpos = .Object, registry = registry())
   count <- tabulate(ids)
   TF <- data.table(
-    id = 0:length(count),
+    id = 0L:length(count),
     count = c(length(which(ids == 0)), count)
   )
   setkeyv(TF, cols = "id")
@@ -384,13 +444,59 @@ setMethod("count", "vector", function(.Object, corpus, p_attribute, ...){
   TF[count > 0]
 })
 
-#' @rdname count-method
-setMethod("count", "Corpus", function(.Object, query = NULL, p_attribute){
-  count(.Object$as.partition(), query = query, p_attribute = p_attribute)
-})
 
+#' @rdname kwic-class
+#' @exportMethod count
+#' @details Applying the \code{count}-method on a \code{kwic} object will return
+#'   a \code{count} object with the evaluation of the left and right context of
+#'   the match.
+#' @examples
+#' # use count-method on kwic object
+#' coi <- kwic("REUTERS", query = "oil") %>%
+#'   count(p_attribute = "word")
+#' 
+#' # features vs cooccurrences-method (identical results)
+#' ref <- count("REUTERS", p_attribute = "word") %>%
+#'   subset(word != "oil")
+#' slot(ref, "size") <- slot(ref, "size") - count("REUTERS", "oil")[["count"]]
+#' y_features <- features(coi, ref, method = "ll", included = TRUE)
+#' y_cooc <- cooccurrences("REUTERS", query = "oil")
+#' 
+setMethod("count", "kwic", function(.Object, p_attribute = "word"){
+  cnt <- .Object@cpos[.Object@cpos[["direction"]] != 0][, .N, by = p_attribute]
+  setnames(cnt, old = "N", new = "count")
+  new(
+    "count",
+    stat = cnt,
+    corpus = .Object@corpus,
+    encoding = .Object@encoding,
+    p_attribute = p_attribute,
+    name = .Object@name,
+    size = sum(cnt[["count"]])
+  )
+})
 
 #' @exportMethod hist
 #' @rdname count_class
 setMethod("hist", "count", function(x, ...) hist(x@stat[,"count"], ...) )
 
+
+#' @rdname count-method
+setMethod("count", "remote_corpus", function(.Object, ...){
+  ocpu_exec(
+    fn = "count",
+    server = .Object@server,
+    user = .Object@user,
+    password = .Object@password,
+    do.call = FALSE,
+    .Object = .Object@corpus,
+    ...
+  )
+})
+
+
+#' @rdname count-method
+setMethod("count", "remote_subcorpus", function(.Object, ...){
+  ocpu_exec(fn = "count", server = .Object@server, do.call = FALSE, .Object = as(.Object, "subcorpus"), ...)
+})
+
