@@ -1,9 +1,12 @@
 library(shiny)
 library(polmineR)
+library(data.table)
 library(magrittr)
 loadNamespace("DT")
 library(shinythemes)
 
+options(polmineR.shiny = TRUE)
+
 source("modules/cooccurrences.R")
 source("modules/dispersion.R")
 source("modules/partition.R")
