@@ -23,9 +23,9 @@ featuresUiInput <- function(){
       selectInput("features_partition_y", "reference partition", choices = character())
     ),
     included = radioButtons("features_included", "included", choices = list("TRUE", "FALSE"), selected = "FALSE", inline = TRUE),
-    pAttribute = selectInput(
-      "features_pAttribute", "pAttribute",
-      choices = pAttributes(corpus()[["corpus"]][1])
+    p_attribute = selectInput(
+      "features_p_attribute", "p_attribute",
+      choices = p_attributes(corpus()[["corpus"]][1])
     )
     
   )
@@ -43,22 +43,37 @@ featuresUiOutput <- function(){
 #' @export featuresServer
 featuresServer <- function(input, output, session){
   
+  observeEvent(
+    input$features_partition_x,
+    {
+      if (input$features_partition_x != ""){
+        updateSelectInput(
+          session,
+          inputId = "features_p_attribute",
+          choices = p_attributes(values[["partitions"]][[input$features_partition_x]])
+        )
+      }
+    }
+  )
+  
+  
   # the sole purpose of the following block is to show empty table
   output$features_table <- DT::renderDataTable({
     input$features_go
     isolate({
       if (input$features_go == 0){
-        return(data.frame(
+        df <- data.frame(
           word = character(), count_coi = integer(), count_ref = integer(),
-          exp_coi = numeric(), chisquare = numeric(), rank_chisquare = integer(), 
-        ))
+          exp_coi = numeric(), chisquare = numeric(), rank_chisquare = integer() 
+        )
+        return(df)
       }
     })
   })
   
   
   
-  retval <- data.frame(a = ""[0], b = ""[0], c = ""[0])
+  retval <- data.frame(a = character(), b = character(), c = character())
   
   observeEvent(
     input$features_go,
@@ -67,8 +82,8 @@ featuresServer <- function(input, output, session){
      
        x = values$partitions[[input$features_partition_x]]
       
-      if (!identical(x@pAttribute, input$features_pAttribute)){
-         x <- enrich(x, pAttribute = input$features_pAttribute)
+      if (!identical(x@p_attribute, input$features_p_attribute)){
+         x <- enrich(x, p_attribute = input$features_p_attribute)
       }
 
       y <- switch(
@@ -78,8 +93,8 @@ featuresServer <- function(input, output, session){
       )
        
       if(input$features_object_y == "partition"){
-        if (!identical(y@pAttribute, input$features_pAttribute)){
-          y <- enrich(y, pAttribute = input$features_pAttribute)
+        if (!identical(y@p_attribute, input$features_p_attribute)){
+          y <- enrich(y, p_attribute = input$features_p_attribute)
         }
       }
       
@@ -101,7 +116,6 @@ featuresServer <- function(input, output, session){
     input$features_table_rows_selected,
     {
       if (length(input$features_table_rows_selected) > 0){
-        print("doing updates")
         updateSelectInput(session, "kwic_object", selected = "partition")
         updateSelectInput(
           session, "kwic_partition",
@@ -111,7 +125,7 @@ featuresServer <- function(input, output, session){
         featuresObject <- values[["features"]]
         token <- featuresObject@stat[["word"]][input$features_table_rows_selected]
         updateTextInput(session, "kwic_query", value = token[1])
-        updateSelectInput(session, "kwic_pAttribute", selected = input$features_pAttribute)
+        updateSelectInput(session, "kwic_p_attribute", selected = input$features_p_attribute)
         updateNavbarPage(session, "polmineR", selected = "kwic")
         Time <- as.character(Sys.time())
         updateSelectInput(session, "kwic_read", choices = Time, selected = Time)
