@@ -14,17 +14,18 @@ kwicUiInput <- function(drop = NULL){
       condition = "input.kwic_object == 'partition'",
       selectInput("kwic_partition", "partition", choices = character())
     ),
-    query = textInput("kwic_query", "query", value = ""),
+    query = textInput("kwic_query", label = "query", value = ""),
     cqp = radioButtons("kwic_cqp", "CQP", choices = list("yes", "no"), selected = "no", inline = TRUE),
-    positivelist = textInput("kwic_positivelist", "positivelist", value = ""),
-    sAttribute = selectInput(
-      "kwic_meta", "sAttribute",
-      choices = sAttributes(corpus()[["corpus"]][1]),
+    positivelist = textInput("kwic_positivelist", label = "positivelist", value = ""),
+    s_attribute = selectInput(
+      "kwic_meta", "s_attribute",
+      choices = s_attributes(corpus()[["corpus"]][1]),
       multiple = TRUE
     ),
-    pAttribute = selectInput(
-      "kwic_pAttribute", "pAttribute",
-      choices = pAttributes(corpus()[["corpus"]][1])
+    p_attribute = selectInput(
+      "kwic_p_attribute", "p_attribute",
+      choices = p_attributes(corpus()[["corpus"]][1]),
+      selected = "word"
     ),
     window = sliderInput("kwic_window", "window", min = 1, max = 25, value = getOption("polmineR.left")),
     br3 = br()
@@ -45,18 +46,18 @@ kwicServer <- function(input, output, session, ...){
   observe({
     x <- input$kwic_partition
     if (x != "" && length(values$partitions) > 0 && input$kwic_partition %in% names(values$partitions)){
-      new_sAttr <- sAttributes(values$partitions[[input$kwic_partition]]@corpus)
-      new_pAttr <- pAttributes(values$partitions[[input$kwic_partition]]@corpus)
-      updateSelectInput(session, "kwic_pAttribute", choices = new_pAttr, selected = NULL)
+      new_sAttr <- s_attributes(values$partitions[[input$kwic_partition]]@corpus)
+      new_pAttr <- p_attributes(values$partitions[[input$kwic_partition]]@corpus)
+      updateSelectInput(session, "kwic_p_attribute", choices = new_pAttr, selected = NULL)
       updateSelectInput(session, "kwic_meta", choices = new_sAttr, selected = NULL)
     }
   })
   
   observe({
     x <- input$kwic_corpus
-    new_sAttr <- sAttributes(input$kwic_corpus)
-    new_pAttr <- pAttributes(input$kwic_corpus)
-    updateSelectInput(session, "kwic_pAttribute", choices = new_pAttr, selected = NULL)
+    new_sAttr <- s_attributes(input$kwic_corpus)
+    new_pAttr <- p_attributes(input$kwic_corpus)
+    updateSelectInput(session, "kwic_p_attribute", choices = new_pAttr, selected = NULL)
     updateSelectInput(session, "kwic_meta", choices = new_sAttr, selected = NULL)
   })
   
@@ -70,53 +71,47 @@ kwicServer <- function(input, output, session, ...){
       
       if ((input$kwic_go > 0 || values[["kwic_go"]] != values[["startingTime"]]) && input$kwic_query != ""){
         
-        if (input$kwic_object == "corpus"){
-          object <- input$kwic_corpus
-        } else {
-          object <- values$partitions[[input$kwic_partition]]
-        }
-        
+        object <- if (input$kwic_object == "corpus")
+          input$kwic_corpus
+        else
+          values$partitions[[input$kwic_partition]]
+
         withProgress(
           message = "please wait", value = 0, max = 5, detail = "preparing data",
           {
-            print("yeah")
+            poslist <- if (nchar(input$kwic_positivelist) >= 1L) input$kwic_positivelist else NULL
             K <- polmineR::kwic(
               .Object = object,
               query = rectifySpecialChars(input$kwic_query),
               cqp = if (input$kwic_cqp == "yes") TRUE else FALSE,
-              pAttribute = if (is.null(input$kwic_pAttribute)) "word" else input$kwic_pAttribute,
+              p_attribute = if (is.null(input$kwic_p_attribute)) "word" else input$kwic_p_attribute,
               left = input$kwic_window,
               right = input$kwic_window,
               meta = input$kwic_meta,
               verbose = "shiny",
-              positivelist = if (nchar(input$kwic_positivelist) > 0) input$kwic_positivelist else NULL,
+              positivelist = poslist,
               cpos = TRUE # required for reading
             )
-            print(K)
+            if (!is.null(poslist) & !is.null(K)) K <- highlight(K, yellow = poslist)
             values[["kwic"]] <- K
           }
         ) # end withProgress
         
         if (is.null(values[["kwic"]])){
-          print("no hits 1")
           retval <- data.frame(left = character(), node = character(), right = character())
-        } else if (nrow(values[["kwic"]]@table) == 0){
-          print("no hits 2")
+        } else if (nrow(values[["kwic"]]@stat) == 0L){
           retval <- data.frame(left = character(), node = character(), right = character())
         } else {
-          print("presumably hits")
-          tab <- values[["kwic"]]@table
-          tab[["hit_no"]] <- NULL
-          if (length(input$kwic_meta) == 0){
-            retval <- data.frame(no = c(1:nrow(tab)), tab)
+          tab <- values[["kwic"]]@stat
+          if ("match_id" %in% colnames(tab)) tab[, "match_id" := NULL]
+          if (length(input$kwic_meta) == 0L){
+            retval <- data.frame(no = 1L:nrow(tab), tab)
           } else if (length(input$kwic_meta)){
             metaRow <- unlist(lapply(
-              c(1:nrow(tab)),
-              function(i){
-                paste(unlist(lapply(tab[i,c(1:length(input$kwic_meta))], as.character)), collapse=" | ")
-              }
+              1L:nrow(tab),
+              function(i)paste(unlist(lapply(tab[i,1L:length(input$kwic_meta)], as.character)), collapse = " | ")
             ))
-            retval <- data.frame(meta = metaRow, tab[,(length(input$kwic_meta)+1):ncol(tab)])
+            retval <- data.frame(meta = metaRow, tab[,(length(input$kwic_meta) + 1L):ncol(tab)])
           }
         }
         
@@ -128,7 +123,7 @@ kwicServer <- function(input, output, session, ...){
     })
     
     # format DataTable
-    retval <- DT::datatable(retval, selection = "single", rownames = FALSE)
+    retval <- DT::datatable(retval, selection = "single", rownames = FALSE, escape = FALSE)
     retval <- DT::formatStyle(retval, "node", color = "#4863A0", textAlign = "center")
     retval <- DT::formatStyle(retval, "left", textAlign = "right")
     if (length(input$kwic_meta) > 0){
@@ -163,11 +158,12 @@ kwicServer <- function(input, output, session, ...){
   observeEvent(
     input$kwic_mail,
     {
-      if (input$kwic_mail > 0) polmineR:::mail(values[["kwic"]])
+      success <- polmineR::mail(values[["kwic"]])
+      if (success) showNotification(
+        sprintf("Results have been sent successfully to %s", getOption("polmineR.email"))
+      )
     }
   )
-  
-  
 }
 
 
@@ -186,7 +182,7 @@ setMethod("kwic", "missing", function(){
       ),
       div(br()),
       sidebarLayout(
-        sidebarPanel = sidebarPanel(kwicUiInput(drop = c("go", "br1", "br2", "pAttribute", "read"))),
+        sidebarPanel = sidebarPanel(kwicUiInput(drop = c("go", "br1", "br2", "p_attribute", "read"))),
         mainPanel = mainPanel(kwicUiOutput())
       )
     ))
